/*
 * Copyright LWJGL. All rights reserved.
 * License terms: https://www.lwjgl.org/license
 * MACHINE GENERATED FILE, DO NOT EDIT
 */
package org.lwjgl.openvr;

import javax.annotation.*;

import java.nio.*;

import org.lwjgl.*;
import org.lwjgl.system.*;

import static org.lwjgl.system.MemoryUtil.*;
import static org.lwjgl.system.MemoryStack.*;

/**
 * Contains information about one axis on the controller.
 * 
 * <h3>Layout</h3>
 * 
 * <pre><code>
 * struct VRControllerAxis_t {
 *     float {@link #x};
 *     float {@link #y};
 * }</code></pre>
 */
@NativeType("struct VRControllerAxis_t")
public class VRControllerAxis extends Struct<VRControllerAxis> implements NativeResource {

    /** The struct size in bytes. */
    public static final int SIZEOF;

    /** The struct alignment in bytes. */
    public static final int ALIGNOF;

    /** The struct member offsets. */
    public static final int
        X,
        Y;

    static {
        Layout layout = __struct(
            __member(4),
            __member(4)
        );

        SIZEOF = layout.getSize();
        ALIGNOF = layout.getAlignment();

        X = layout.offsetof(0);
        Y = layout.offsetof(1);
    }

    protected VRControllerAxis(long address, @Nullable ByteBuffer container) {
        super(address, container);
    }

    @Override
    protected VRControllerAxis create(long address, @Nullable ByteBuffer container) {
        return new VRControllerAxis(address, container);
    }

    /**
     * Creates a {@code VRControllerAxis} instance at the current position of the specified {@link ByteBuffer} container. Changes to the buffer's content will be
     * visible to the struct instance and vice versa.
     *
     * <p>The created instance holds a strong reference to the container object.</p>
     */
    public VRControllerAxis(ByteBuffer container) {
        super(memAddress(container), __checkContainer(container, SIZEOF));
    }

    @Override
    public int sizeof() { return SIZEOF; }

    /** Ranges from -1.0 to 1.0 for joysticks and track pads. Ranges from 0.0 to 1.0 for triggers were 0 is fully released. */
    public float x() { return nx(address()); }
    /** Ranges from -1.0 to 1.0 for joysticks and track pads. Is always 0.0 for triggers. */
    public float y() { return ny(address()); }

    /** Sets the specified value to the {@link #x} field. */
    public VRControllerAxis x(float value) { nx(address(), value); return this; }
    /** Sets the specified value to the {@link #y} field. */
    public VRControllerAxis y(float value) { ny(address(), value); return this; }

    /** Initializes this struct with the specified values. */
    public VRControllerAxis set(
        float x,
        float y
    ) {
        x(x);
        y(y);

        return this;
    }

    /**
     * Copies the specified struct data to this struct.
     *
     * @param src the source struct
     *
     * @return this struct
     */
    public VRControllerAxis set(VRControllerAxis src) {
        memCopy(src.address(), address(), SIZEOF);
        return this;
    }

    // -----------------------------------

    /** Returns a new {@code VRControllerAxis} instance allocated with {@link MemoryUtil#memAlloc memAlloc}. The instance must be explicitly freed. */
    public static VRControllerAxis malloc() {
        return new VRControllerAxis(nmemAllocChecked(SIZEOF), null);
    }

    /** Returns a new {@code VRControllerAxis} instance allocated with {@link MemoryUtil#memCalloc memCalloc}. The instance must be explicitly freed. */
    public static VRControllerAxis calloc() {
        return new VRControllerAxis(nmemCallocChecked(1, SIZEOF), null);
    }

    /** Returns a new {@code VRControllerAxis} instance allocated with {@link BufferUtils}. */
    public static VRControllerAxis create() {
        ByteBuffer container = BufferUtils.createByteBuffer(SIZEOF);
        return new VRControllerAxis(memAddress(container), container);
    }

    /** Returns a new {@code VRControllerAxis} instance for the specified memory address. */
    public static VRControllerAxis create(long address) {
        return new VRControllerAxis(address, null);
    }

    /** Like {@link #create(long) create}, but returns {@code null} if {@code address} is {@code NULL}. */
    @Nullable
    public static VRControllerAxis createSafe(long address) {
        return address == NULL ? null : new VRControllerAxis(address, null);
    }

    /**
     * Returns a new {@link VRControllerAxis.Buffer} instance allocated with {@link MemoryUtil#memAlloc memAlloc}. The instance must be explicitly freed.
     *
     * @param capacity the buffer capacity
     */
    public static VRControllerAxis.Buffer malloc(int capacity) {
        return new Buffer(nmemAllocChecked(__checkMalloc(capacity, SIZEOF)), capacity);
    }

    /**
     * Returns a new {@link VRControllerAxis.Buffer} instance allocated with {@link MemoryUtil#memCalloc memCalloc}. The instance must be explicitly freed.
     *
     * @param capacity the buffer capacity
     */
    public static VRControllerAxis.Buffer calloc(int capacity) {
        return new Buffer(nmemCallocChecked(capacity, SIZEOF), capacity);
    }

    /**
     * Returns a new {@link VRControllerAxis.Buffer} instance allocated with {@link BufferUtils}.
     *
     * @param capacity the buffer capacity
     */
    public static VRControllerAxis.Buffer create(int capacity) {
        ByteBuffer container = __create(capacity, SIZEOF);
        return new Buffer(memAddress(container), container, -1, 0, capacity, capacity);
    }

    /**
     * Create a {@link VRControllerAxis.Buffer} instance at the specified memory.
     *
     * @param address  the memory address
     * @param capacity the buffer capacity
     */
    public static VRControllerAxis.Buffer create(long address, int capacity) {
        return new Buffer(address, capacity);
    }

    /** Like {@link #create(long, int) create}, but returns {@code null} if {@code address} is {@code NULL}. */
    @Nullable
    public static VRControllerAxis.Buffer createSafe(long address, int capacity) {
        return address == NULL ? null : new Buffer(address, capacity);
    }

    // -----------------------------------

    /** Deprecated for removal in 3.4.0. Use {@link #malloc(MemoryStack)} instead. */
    @Deprecated public static VRControllerAxis mallocStack() { return malloc(stackGet()); }
    /** Deprecated for removal in 3.4.0. Use {@link #calloc(MemoryStack)} instead. */
    @Deprecated public static VRControllerAxis callocStack() { return calloc(stackGet()); }
    /** Deprecated for removal in 3.4.0. Use {@link #malloc(MemoryStack)} instead. */
    @Deprecated public static VRControllerAxis mallocStack(MemoryStack stack) { return malloc(stack); }
    /** Deprecated for removal in 3.4.0. Use {@link #calloc(MemoryStack)} instead. */
    @Deprecated public static VRControllerAxis callocStack(MemoryStack stack) { return calloc(stack); }
    /** Deprecated for removal in 3.4.0. Use {@link #malloc(int, MemoryStack)} instead. */
    @Deprecated public static VRControllerAxis.Buffer mallocStack(int capacity) { return malloc(capacity, stackGet()); }
    /** Deprecated for removal in 3.4.0. Use {@link #calloc(int, MemoryStack)} instead. */
    @Deprecated public static VRControllerAxis.Buffer callocStack(int capacity) { return calloc(capacity, stackGet()); }
    /** Deprecated for removal in 3.4.0. Use {@link #malloc(int, MemoryStack)} instead. */
    @Deprecated public static VRControllerAxis.Buffer mallocStack(int capacity, MemoryStack stack) { return malloc(capacity, stack); }
    /** Deprecated for removal in 3.4.0. Use {@link #calloc(int, MemoryStack)} instead. */
    @Deprecated public static VRControllerAxis.Buffer callocStack(int capacity, MemoryStack stack) { return calloc(capacity, stack); }

    /**
     * Returns a new {@code VRControllerAxis} instance allocated on the specified {@link MemoryStack}.
     *
     * @param stack the stack from which to allocate
     */
    public static VRControllerAxis malloc(MemoryStack stack) {
        return new VRControllerAxis(stack.nmalloc(ALIGNOF, SIZEOF), null);
    }

    /**
     * Returns a new {@code VRControllerAxis} instance allocated on the specified {@link MemoryStack} and initializes all its bits to zero.
     *
     * @param stack the stack from which to allocate
     */
    public static VRControllerAxis calloc(MemoryStack stack) {
        return new VRControllerAxis(stack.ncalloc(ALIGNOF, 1, SIZEOF), null);
    }

    /**
     * Returns a new {@link VRControllerAxis.Buffer} instance allocated on the specified {@link MemoryStack}.
     *
     * @param stack    the stack from which to allocate
     * @param capacity the buffer capacity
     */
    public static VRControllerAxis.Buffer malloc(int capacity, MemoryStack stack) {
        return new Buffer(stack.nmalloc(ALIGNOF, capacity * SIZEOF), capacity);
    }

    /**
     * Returns a new {@link VRControllerAxis.Buffer} instance allocated on the specified {@link MemoryStack} and initializes all its bits to zero.
     *
     * @param stack    the stack from which to allocate
     * @param capacity the buffer capacity
     */
    public static VRControllerAxis.Buffer calloc(int capacity, MemoryStack stack) {
        return new Buffer(stack.ncalloc(ALIGNOF, capacity, SIZEOF), capacity);
    }

    // -----------------------------------

    /** Unsafe version of {@link #x}. */
    public static float nx(long struct) { return UNSAFE.getFloat(null, struct + VRControllerAxis.X); }
    /** Unsafe version of {@link #y}. */
    public static float ny(long struct) { return UNSAFE.getFloat(null, struct + VRControllerAxis.Y); }

    /** Unsafe version of {@link #x(float) x}. */
    public static void nx(long struct, float value) { UNSAFE.putFloat(null, struct + VRControllerAxis.X, value); }
    /** Unsafe version of {@link #y(float) y}. */
    public static void ny(long struct, float value) { UNSAFE.putFloat(null, struct + VRControllerAxis.Y, value); }

    // -----------------------------------

    /** An array of {@link VRControllerAxis} structs. */
    public static class Buffer extends StructBuffer<VRControllerAxis, Buffer> implements NativeResource {

        private static final VRControllerAxis ELEMENT_FACTORY = VRControllerAxis.create(-1L);

        /**
         * Creates a new {@code VRControllerAxis.Buffer} instance backed by the specified container.
         *
         * <p>Changes to the container's content will be visible to the struct buffer instance and vice versa. The two buffers' position, limit, and mark values
         * will be independent. The new buffer's position will be zero, its capacity and its limit will be the number of bytes remaining in this buffer divided
         * by {@link VRControllerAxis#SIZEOF}, and its mark will be undefined.</p>
         *
         * <p>The created buffer instance holds a strong reference to the container object.</p>
         */
        public Buffer(ByteBuffer container) {
            super(container, container.remaining() / SIZEOF);
        }

        public Buffer(long address, int cap) {
            super(address, null, -1, 0, cap, cap);
        }

        Buffer(long address, @Nullable ByteBuffer container, int mark, int pos, int lim, int cap) {
            super(address, container, mark, pos, lim, cap);
        }

        @Override
        protected Buffer self() {
            return this;
        }

        @Override
        protected VRControllerAxis getElementFactory() {
            return ELEMENT_FACTORY;
        }

        /** @return the value of the {@link VRControllerAxis#x} field. */
        public float x() { return VRControllerAxis.nx(address()); }
        /** @return the value of the {@link VRControllerAxis#y} field. */
        public float y() { return VRControllerAxis.ny(address()); }

        /** Sets the specified value to the {@link VRControllerAxis#x} field. */
        public VRControllerAxis.Buffer x(float value) { VRControllerAxis.nx(address(), value); return this; }
        /** Sets the specified value to the {@link VRControllerAxis#y} field. */
        public VRControllerAxis.Buffer y(float value) { VRControllerAxis.ny(address(), value); return this; }

    }

}