/*
 * Copyright LWJGL. All rights reserved.
 * License terms: https://www.lwjgl.org/license
 * MACHINE GENERATED FILE, DO NOT EDIT
 */
package org.lwjgl.openxr;

import javax.annotation.*;

import java.nio.*;

import org.lwjgl.*;
import org.lwjgl.system.*;

import static org.lwjgl.system.MemoryUtil.*;
import static org.lwjgl.system.MemoryStack.*;

/**
 * Information to create a marker detection handle for Aruco markers.
 * 
 * <h5>Description</h5>
 * 
 * <p>This structure is required by the {@link MLMarkerUnderstanding#XR_MARKER_TYPE_ARUCO_ML MARKER_TYPE_ARUCO_ML} detector.</p>
 * 
 * <h5>Valid Usage (Implicit)</h5>
 * 
 * <ul>
 * <li>The {@link MLMarkerUnderstanding XR_ML_marker_understanding} extension <b>must</b> be enabled prior to using {@link XrMarkerDetectorArucoInfoML}</li>
 * <li>{@code type} <b>must</b> be {@link MLMarkerUnderstanding#XR_TYPE_MARKER_DETECTOR_ARUCO_INFO_ML TYPE_MARKER_DETECTOR_ARUCO_INFO_ML}</li>
 * <li>{@code next} <b>must</b> be {@code NULL} or a valid pointer to the <a href="https://registry.khronos.org/OpenXR/specs/1.0/html/xrspec.html#valid-usage-for-structure-pointer-chains">next structure in a structure chain</a></li>
 * <li>{@code arucoDict} <b>must</b> be a valid {@code XrMarkerArucoDictML} value</li>
 * </ul>
 * 
 * <h5>See Also</h5>
 * 
 * <p>{@link XrMarkerDetectorCreateInfoML}, {@link MLMarkerUnderstanding#xrCreateMarkerDetectorML CreateMarkerDetectorML}</p>
 * 
 * <h3>Layout</h3>
 * 
 * <pre><code>
 * struct XrMarkerDetectorArucoInfoML {
 *     XrStructureType {@link #type};
 *     void const * {@link #next};
 *     XrMarkerArucoDictML {@link #arucoDict};
 * }</code></pre>
 */
public class XrMarkerDetectorArucoInfoML extends Struct<XrMarkerDetectorArucoInfoML> implements NativeResource {

    /** The struct size in bytes. */
    public static final int SIZEOF;

    /** The struct alignment in bytes. */
    public static final int ALIGNOF;

    /** The struct member offsets. */
    public static final int
        TYPE,
        NEXT,
        ARUCODICT;

    static {
        Layout layout = __struct(
            __member(4),
            __member(POINTER_SIZE),
            __member(4)
        );

        SIZEOF = layout.getSize();
        ALIGNOF = layout.getAlignment();

        TYPE = layout.offsetof(0);
        NEXT = layout.offsetof(1);
        ARUCODICT = layout.offsetof(2);
    }

    protected XrMarkerDetectorArucoInfoML(long address, @Nullable ByteBuffer container) {
        super(address, container);
    }

    @Override
    protected XrMarkerDetectorArucoInfoML create(long address, @Nullable ByteBuffer container) {
        return new XrMarkerDetectorArucoInfoML(address, container);
    }

    /**
     * Creates a {@code XrMarkerDetectorArucoInfoML} instance at the current position of the specified {@link ByteBuffer} container. Changes to the buffer's content will be
     * visible to the struct instance and vice versa.
     *
     * <p>The created instance holds a strong reference to the container object.</p>
     */
    public XrMarkerDetectorArucoInfoML(ByteBuffer container) {
        super(memAddress(container), __checkContainer(container, SIZEOF));
    }

    @Override
    public int sizeof() { return SIZEOF; }

    /** the {@code XrStructureType} of this structure. */
    @NativeType("XrStructureType")
    public int type() { return ntype(address()); }
    /** {@code NULL} or a pointer to the next structure in a structure chain. */
    @NativeType("void const *")
    public long next() { return nnext(address()); }
    /** the ArUco dictionary name from which markers will be detected. */
    @NativeType("XrMarkerArucoDictML")
    public int arucoDict() { return narucoDict(address()); }

    /** Sets the specified value to the {@link #type} field. */
    public XrMarkerDetectorArucoInfoML type(@NativeType("XrStructureType") int value) { ntype(address(), value); return this; }
    /** Sets the {@link MLMarkerUnderstanding#XR_TYPE_MARKER_DETECTOR_ARUCO_INFO_ML TYPE_MARKER_DETECTOR_ARUCO_INFO_ML} value to the {@link #type} field. */
    public XrMarkerDetectorArucoInfoML type$Default() { return type(MLMarkerUnderstanding.XR_TYPE_MARKER_DETECTOR_ARUCO_INFO_ML); }
    /** Sets the specified value to the {@link #next} field. */
    public XrMarkerDetectorArucoInfoML next(@NativeType("void const *") long value) { nnext(address(), value); return this; }
    /** Sets the specified value to the {@link #arucoDict} field. */
    public XrMarkerDetectorArucoInfoML arucoDict(@NativeType("XrMarkerArucoDictML") int value) { narucoDict(address(), value); return this; }

    /** Initializes this struct with the specified values. */
    public XrMarkerDetectorArucoInfoML set(
        int type,
        long next,
        int arucoDict
    ) {
        type(type);
        next(next);
        arucoDict(arucoDict);

        return this;
    }

    /**
     * Copies the specified struct data to this struct.
     *
     * @param src the source struct
     *
     * @return this struct
     */
    public XrMarkerDetectorArucoInfoML set(XrMarkerDetectorArucoInfoML src) {
        memCopy(src.address(), address(), SIZEOF);
        return this;
    }

    // -----------------------------------

    /** Returns a new {@code XrMarkerDetectorArucoInfoML} instance allocated with {@link MemoryUtil#memAlloc memAlloc}. The instance must be explicitly freed. */
    public static XrMarkerDetectorArucoInfoML malloc() {
        return new XrMarkerDetectorArucoInfoML(nmemAllocChecked(SIZEOF), null);
    }

    /** Returns a new {@code XrMarkerDetectorArucoInfoML} instance allocated with {@link MemoryUtil#memCalloc memCalloc}. The instance must be explicitly freed. */
    public static XrMarkerDetectorArucoInfoML calloc() {
        return new XrMarkerDetectorArucoInfoML(nmemCallocChecked(1, SIZEOF), null);
    }

    /** Returns a new {@code XrMarkerDetectorArucoInfoML} instance allocated with {@link BufferUtils}. */
    public static XrMarkerDetectorArucoInfoML create() {
        ByteBuffer container = BufferUtils.createByteBuffer(SIZEOF);
        return new XrMarkerDetectorArucoInfoML(memAddress(container), container);
    }

    /** Returns a new {@code XrMarkerDetectorArucoInfoML} instance for the specified memory address. */
    public static XrMarkerDetectorArucoInfoML create(long address) {
        return new XrMarkerDetectorArucoInfoML(address, null);
    }

    /** Like {@link #create(long) create}, but returns {@code null} if {@code address} is {@code NULL}. */
    @Nullable
    public static XrMarkerDetectorArucoInfoML createSafe(long address) {
        return address == NULL ? null : new XrMarkerDetectorArucoInfoML(address, null);
    }

    /**
     * Returns a new {@link XrMarkerDetectorArucoInfoML.Buffer} instance allocated with {@link MemoryUtil#memAlloc memAlloc}. The instance must be explicitly freed.
     *
     * @param capacity the buffer capacity
     */
    public static XrMarkerDetectorArucoInfoML.Buffer malloc(int capacity) {
        return new Buffer(nmemAllocChecked(__checkMalloc(capacity, SIZEOF)), capacity);
    }

    /**
     * Returns a new {@link XrMarkerDetectorArucoInfoML.Buffer} instance allocated with {@link MemoryUtil#memCalloc memCalloc}. The instance must be explicitly freed.
     *
     * @param capacity the buffer capacity
     */
    public static XrMarkerDetectorArucoInfoML.Buffer calloc(int capacity) {
        return new Buffer(nmemCallocChecked(capacity, SIZEOF), capacity);
    }

    /**
     * Returns a new {@link XrMarkerDetectorArucoInfoML.Buffer} instance allocated with {@link BufferUtils}.
     *
     * @param capacity the buffer capacity
     */
    public static XrMarkerDetectorArucoInfoML.Buffer create(int capacity) {
        ByteBuffer container = __create(capacity, SIZEOF);
        return new Buffer(memAddress(container), container, -1, 0, capacity, capacity);
    }

    /**
     * Create a {@link XrMarkerDetectorArucoInfoML.Buffer} instance at the specified memory.
     *
     * @param address  the memory address
     * @param capacity the buffer capacity
     */
    public static XrMarkerDetectorArucoInfoML.Buffer create(long address, int capacity) {
        return new Buffer(address, capacity);
    }

    /** Like {@link #create(long, int) create}, but returns {@code null} if {@code address} is {@code NULL}. */
    @Nullable
    public static XrMarkerDetectorArucoInfoML.Buffer createSafe(long address, int capacity) {
        return address == NULL ? null : new Buffer(address, capacity);
    }

    /**
     * Returns a new {@code XrMarkerDetectorArucoInfoML} instance allocated on the specified {@link MemoryStack}.
     *
     * @param stack the stack from which to allocate
     */
    public static XrMarkerDetectorArucoInfoML malloc(MemoryStack stack) {
        return new XrMarkerDetectorArucoInfoML(stack.nmalloc(ALIGNOF, SIZEOF), null);
    }

    /**
     * Returns a new {@code XrMarkerDetectorArucoInfoML} instance allocated on the specified {@link MemoryStack} and initializes all its bits to zero.
     *
     * @param stack the stack from which to allocate
     */
    public static XrMarkerDetectorArucoInfoML calloc(MemoryStack stack) {
        return new XrMarkerDetectorArucoInfoML(stack.ncalloc(ALIGNOF, 1, SIZEOF), null);
    }

    /**
     * Returns a new {@link XrMarkerDetectorArucoInfoML.Buffer} instance allocated on the specified {@link MemoryStack}.
     *
     * @param stack    the stack from which to allocate
     * @param capacity the buffer capacity
     */
    public static XrMarkerDetectorArucoInfoML.Buffer malloc(int capacity, MemoryStack stack) {
        return new Buffer(stack.nmalloc(ALIGNOF, capacity * SIZEOF), capacity);
    }

    /**
     * Returns a new {@link XrMarkerDetectorArucoInfoML.Buffer} instance allocated on the specified {@link MemoryStack} and initializes all its bits to zero.
     *
     * @param stack    the stack from which to allocate
     * @param capacity the buffer capacity
     */
    public static XrMarkerDetectorArucoInfoML.Buffer calloc(int capacity, MemoryStack stack) {
        return new Buffer(stack.ncalloc(ALIGNOF, capacity, SIZEOF), capacity);
    }

    // -----------------------------------

    /** Unsafe version of {@link #type}. */
    public static int ntype(long struct) { return UNSAFE.getInt(null, struct + XrMarkerDetectorArucoInfoML.TYPE); }
    /** Unsafe version of {@link #next}. */
    public static long nnext(long struct) { return memGetAddress(struct + XrMarkerDetectorArucoInfoML.NEXT); }
    /** Unsafe version of {@link #arucoDict}. */
    public static int narucoDict(long struct) { return UNSAFE.getInt(null, struct + XrMarkerDetectorArucoInfoML.ARUCODICT); }

    /** Unsafe version of {@link #type(int) type}. */
    public static void ntype(long struct, int value) { UNSAFE.putInt(null, struct + XrMarkerDetectorArucoInfoML.TYPE, value); }
    /** Unsafe version of {@link #next(long) next}. */
    public static void nnext(long struct, long value) { memPutAddress(struct + XrMarkerDetectorArucoInfoML.NEXT, value); }
    /** Unsafe version of {@link #arucoDict(int) arucoDict}. */
    public static void narucoDict(long struct, int value) { UNSAFE.putInt(null, struct + XrMarkerDetectorArucoInfoML.ARUCODICT, value); }

    // -----------------------------------

    /** An array of {@link XrMarkerDetectorArucoInfoML} structs. */
    public static class Buffer extends StructBuffer<XrMarkerDetectorArucoInfoML, Buffer> implements NativeResource {

        private static final XrMarkerDetectorArucoInfoML ELEMENT_FACTORY = XrMarkerDetectorArucoInfoML.create(-1L);

        /**
         * Creates a new {@code XrMarkerDetectorArucoInfoML.Buffer} instance backed by the specified container.
         *
         * <p>Changes to the container's content will be visible to the struct buffer instance and vice versa. The two buffers' position, limit, and mark values
         * will be independent. The new buffer's position will be zero, its capacity and its limit will be the number of bytes remaining in this buffer divided
         * by {@link XrMarkerDetectorArucoInfoML#SIZEOF}, and its mark will be undefined.</p>
         *
         * <p>The created buffer instance holds a strong reference to the container object.</p>
         */
        public Buffer(ByteBuffer container) {
            super(container, container.remaining() / SIZEOF);
        }

        public Buffer(long address, int cap) {
            super(address, null, -1, 0, cap, cap);
        }

        Buffer(long address, @Nullable ByteBuffer container, int mark, int pos, int lim, int cap) {
            super(address, container, mark, pos, lim, cap);
        }

        @Override
        protected Buffer self() {
            return this;
        }

        @Override
        protected XrMarkerDetectorArucoInfoML getElementFactory() {
            return ELEMENT_FACTORY;
        }

        /** @return the value of the {@link XrMarkerDetectorArucoInfoML#type} field. */
        @NativeType("XrStructureType")
        public int type() { return XrMarkerDetectorArucoInfoML.ntype(address()); }
        /** @return the value of the {@link XrMarkerDetectorArucoInfoML#next} field. */
        @NativeType("void const *")
        public long next() { return XrMarkerDetectorArucoInfoML.nnext(address()); }
        /** @return the value of the {@link XrMarkerDetectorArucoInfoML#arucoDict} field. */
        @NativeType("XrMarkerArucoDictML")
        public int arucoDict() { return XrMarkerDetectorArucoInfoML.narucoDict(address()); }

        /** Sets the specified value to the {@link XrMarkerDetectorArucoInfoML#type} field. */
        public XrMarkerDetectorArucoInfoML.Buffer type(@NativeType("XrStructureType") int value) { XrMarkerDetectorArucoInfoML.ntype(address(), value); return this; }
        /** Sets the {@link MLMarkerUnderstanding#XR_TYPE_MARKER_DETECTOR_ARUCO_INFO_ML TYPE_MARKER_DETECTOR_ARUCO_INFO_ML} value to the {@link XrMarkerDetectorArucoInfoML#type} field. */
        public XrMarkerDetectorArucoInfoML.Buffer type$Default() { return type(MLMarkerUnderstanding.XR_TYPE_MARKER_DETECTOR_ARUCO_INFO_ML); }
        /** Sets the specified value to the {@link XrMarkerDetectorArucoInfoML#next} field. */
        public XrMarkerDetectorArucoInfoML.Buffer next(@NativeType("void const *") long value) { XrMarkerDetectorArucoInfoML.nnext(address(), value); return this; }
        /** Sets the specified value to the {@link XrMarkerDetectorArucoInfoML#arucoDict} field. */
        public XrMarkerDetectorArucoInfoML.Buffer arucoDict(@NativeType("XrMarkerArucoDictML") int value) { XrMarkerDetectorArucoInfoML.narucoDict(address(), value); return this; }

    }

}